﻿using CarRepairShopContracts.BindingModels;
using CarRepairShopContracts.BusinessLogicsContracts;
using CarRepairShopContracts.SearchModels;
using CarRepairShopContracts.ViewModels;
using CarRepairShopDataModels.Enums;
using DocumentFormat.OpenXml.Bibliography;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CarRepairShopBusinessLogic.BusinessLogics
{
    public class WorkModeling : IWorkProcess
    {
        private readonly ILogger _logger;
        private readonly Random _rnd;
        private IOrderLogic? _orderLogic;

        public WorkModeling(ILogger<WorkModeling> logger)
        {
            _logger = logger;
            _rnd = new Random(1000);
        }

        public void DoWork(IImplementerLogic implementerLogic, IOrderLogic orderLogic)
        {
            _orderLogic = orderLogic;
            var implementers = implementerLogic.ReadList(null);
            if(implementers == null)
            {
                _logger.LogWarning("DoWork. Исполнителей нет");
                return;
            }
            var orders = _orderLogic.ReadList(new OrderSearchModel
            {
                Status = OrderStatus.Принят
            });
            if(orders == null || orders.Count == 0)
            {
                _logger.LogWarning("DoWork. Заказы пусты или null");
                return;
            }
            _logger.LogDebug("DoWork for {Count} orders", orders.Count);
            foreach(var implementer in implementers)
            {
                Task.Run(() => WorkerWorkAsync(implementer, orders));
            }
        }

        private async Task WorkerWorkAsync(ImplementerViewModel implementer, List<OrderViewModel> orders)
        {
            if(_orderLogic == null || implementer == null)
            {
                return;
            }
            await RunOrderInWork(implementer);

            await Task.Run(() =>
            {
                foreach (var order in orders)
                {
                    try
                    {
                        _logger.LogDebug("DoWork. Исполнитель {Id} пытается взять заказ {order}", implementer.Id, order.Id);
                        _orderLogic.TakeOrderInWork(new OrderBindingModel
                        {
                            Id = order.Id,
                            ImplementerId = implementer.Id
                        });

                        Thread.Sleep(implementer.WorkExperience * _rnd.Next(100, 1000) * order.Count);
                        _logger.LogDebug("DoWork. Исполнитель {Id} закончил заказ {Order}", implementer.Id, order.Id);
                        _orderLogic.FinishOrder(new OrderBindingModel
                        {
                            Id = order.Id,
                        });
                    }
                    catch (InvalidOperationException ex)
                    {
                        _logger.LogWarning(ex, "Ошибка с получением работы");
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError(ex, "Ошибка во время работы");
                        throw;
                    }
                    Thread.Sleep(implementer.Qualification * _rnd.Next(10, 100));
                }
            });
        }

        private async Task RunOrderInWork(ImplementerViewModel implementer)
        {
            if(_orderLogic == null || implementer == null)
            {
                return;
            }
            try
            {
                var runOrder = await Task.Run(() => _orderLogic.ReadElement(new OrderSearchModel
                {
                    ImplementerId = implementer.Id,
                    Status = OrderStatus.Выполняется
                }));
                if(runOrder == null)
                {
                    return;
                }

                _logger.LogDebug("DoWork. Исполнитель {Id} вернулся к заказу {Order}", implementer.Id, runOrder.Id);

                Thread.Sleep(implementer.WorkExperience * _rnd.Next(100, 300) * runOrder.Count);
                
                _logger.LogDebug("DoWork. Исполнитель {Id} закончил заказ {Order}", implementer.Id, runOrder.Id);
                _orderLogic.FinishOrder(new OrderBindingModel
                {
                    Id = implementer.Id
                });
                Thread.Sleep(implementer.Qualification * _rnd.Next(10, 100));
            }
            catch(InvalidOperationException ex)
            {
                _logger.LogWarning(ex, "Ошибка получения работы");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка во время работы");
                throw;
            }

        }
    }
}
